﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Olap.FlatData;
using System.Linq.Expressions;
using Infragistics.Olap.Xmla;
using Infragistics.Olap;
using Infragistics.DragDrop;
using Infragistics.Controls.Grids;

namespace XamPivotGridCustomSort
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        PivotPartType ppt;
        public MainWindow()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(MainWindow_Loaded);
            xamPivotGrid1.PivotItemDragEnter += xamPivotGrid1_PivotItemDragEnter;
            
        }

        void xamPivotGrid1_PivotItemDragEnter(object sender, Infragistics.Controls.Grids.PivotItemDragDropEventArgs e)
        {
            ppt = e.PivotPartType;
            // change Delete action areas to None action
            if (e.PivotPartType == PivotPartType.DeleteArea)
            {
                XamPivotGrid.SetPivotPartType(e.DragDropEventArgs.DropTarget, PivotPartType.None);
            }

            // cheat drag-drop framework to show drop-not-allowed currsor
            if (XamPivotGrid.GetPivotPartType(e.DragDropEventArgs.DropTarget) == PivotPartType.None)
            {
                e.DragDropEventArgs.MoveCursorTemplate = DragDropManager.CurrentDropNotAllowedCursorTemplate;
                e.DragDropEventArgs.OperationType = OperationType.Move;
            }
        }


        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            this.LoadPivotGridWithSampleData();
            xamPivotGrid1.DataSource.Rows.CollectionChanging += Rows_CollectionChanging;
            xamPivotGrid1.DataSource.Columns.CollectionChanging += Columns_CollectionChanging;
            xamPivotGrid1.DataSource.Measures.CollectionChanging += Measures_CollectionChanging;
            xamPivotGrid1.DataSource.Filters.CollectionChanging += Filters_CollectionChanging;
        }

        void Filters_CollectionChanging(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            SwapCollection<IAreaItemViewModel> swapCollection = (SwapCollection<IAreaItemViewModel>)sender;
            if (ppt != PivotPartType.Filters)
            {
                swapCollection.CancelCollectionChanging = true;
            }
            else
            swapCollection.CancelCollectionChanging = !this.xamPivotGrid1.DataSource.IsSwitchingAreaItemLocation;
        }

        void Measures_CollectionChanging(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            SwapCollection<IAreaItemViewModel> swapCollection = (SwapCollection<IAreaItemViewModel>)sender;
            if (ppt != PivotPartType.Measures)
            {
                swapCollection.CancelCollectionChanging = true;
            }
            else
            swapCollection.CancelCollectionChanging = !this.xamPivotGrid1.DataSource.IsSwitchingAreaItemLocation;
        }

        void Columns_CollectionChanging(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            SwapCollection<IAreaItemViewModel> swapCollection = (SwapCollection<IAreaItemViewModel>)sender;
            if (ppt != PivotPartType.Columns)
            {
                swapCollection.CancelCollectionChanging = true;
            }
            else
            swapCollection.CancelCollectionChanging = !this.xamPivotGrid1.DataSource.IsSwitchingAreaItemLocation;
        }

        void Rows_CollectionChanging(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            SwapCollection<IAreaItemViewModel> swapCollection = (SwapCollection<IAreaItemViewModel>)sender;
            if (ppt != PivotPartType.Rows)
            {
                swapCollection.CancelCollectionChanging = true;
            }
            else
            swapCollection.CancelCollectionChanging = !this.xamPivotGrid1.DataSource.IsSwitchingAreaItemLocation;
        }

        List<PartnerPercentage> myList;

        private void LoadPivotGridWithSampleData()
        {
            try
            {
                //populating some data
                myList = new List<PartnerPercentage>();
                for (int i = 0; i < 30; i++)
                {
                    for (int j = 0; j < 100; j++)
                    {
                        if (i % 4 == 0)
                        {
                            myList.Insert(0, new PartnerPercentage { Investment = new InvestmentType() { Order = i + 1, Name = string.Format("{0} - 10", i) }, Partner = "P " + j + " " + i, Percentage = (double)j / 100 });
                        }
                        else
                        {
                            myList.Add(new PartnerPercentage { Investment = new InvestmentType() { Order = i + 1, Name = string.Format("{0} - 10", i) }, Partner = "P " + j + " " + i, Percentage = (double)j / 100 });
                        }
                    }
                }

                //define flatDataSource based on the hierarchies
                FlatDataSource flatDataSource = new FlatDataSource
                {
                    ItemsSource = myList,
                    Cube = XmlaDataSource.GenerateInitialCube("PartnerPercentage"),
                    Rows = XmlaDataSource.GenerateInitialItems("[Partner].[Shown Partner Header]"),
                    Columns = XmlaDataSource.GenerateInitialItems("[Investment].[Investment]"),
                    Measures = XmlaDataSource.GenerateInitialItems("Percentage"),
                };

                // Define hierarchies
                HierarchyDescriptor<PartnerPercentage> partnerHierarchy = new HierarchyDescriptor<PartnerPercentage>(p => p.Partner);
                partnerHierarchy.HierarchyName = "Shown Partner Header";
                partnerHierarchy.AddLevel(p =>
                    "all partners",
                    "All adaa  partners"
                    );
                partnerHierarchy.AddLevel(p =>
                    p.Partner,
                    "Partner name"
                    );

                Expression<Func<PartnerPercentage, int>> expression1 =
                      p => int.Parse(p.Partner.Split(' ')[2]);
                partnerHierarchy.LevelDescriptors[1].OrderByKeyExpression = expression1;

                flatDataSource.AddHierarchyDescriptor(partnerHierarchy);

                HierarchyDescriptor<PartnerPercentage> investmentHierarchy = new HierarchyDescriptor<PartnerPercentage>(p => p.Investment);
                investmentHierarchy.AddLevel(p =>
                    "All investments",
                    "All aaa investments"
                    );

                investmentHierarchy.AddLevel(p =>
                    p.Investment.Order,
                    "Order"
                    );

                //ADD SORT ORDER

                flatDataSource.PreserveMembersOrder = false;

                char[] separators = { ' ' };

                Expression<Func<PartnerPercentage, int>> expression =
                        p => p.Investment.Order;
                investmentHierarchy.LevelDescriptors[1].OrderByKeyExpression = expression;


                flatDataSource.AddHierarchyDescriptor(investmentHierarchy);

                flatDataSource.LevelDefaultSortDirections.Add(new LevelSortDirection { LevelUniqueName = "[Investment].[Investment].[Order]", SortBehavior = LevelSortBehavior.System });

                this.xamPivotGrid1.DataSource = flatDataSource;
            }
            catch (Exception ex)
            {
                MessageBox.Show("LoadPivotGridWithSampleData - " + ex.Message);
            }
        }
    }

    public class InvestmentType
    {
        public int Order { get; set; }

        public string Name { get; set; }
    }

    public class PartnerPercentage
    {
        public InvestmentType Investment { get; set; }
        public string Partner { get; set; }
        public double Percentage { get; set; }
    }
}
